/** \file clamp_test.h
 * \brief Class to test the effect of clamping a single variable
 * 
 * \todo This class is a work in progress, please don't depend on it
 *
 * Runs an InfAlg such as BP with specified variable clamped to specified
 * state, and clamped to every other state; and combines the results
 * using the logZ estimates.
 * 
 * Useful to see which variable/state pairs have best reduction in error
 * upon clamping
 * 
 * Also supports clamping of factors
 */

#ifndef ____defined_libdai_clamp_test_h__
#define ____defined_libdai_clamp_test_h__

#include <dai/daialg.h>

#include <dai/clamp_test.h>
#include <dai/bbp.h>

namespace dai {

using namespace std;

/// Class to test the effect of clamping a single variable
class ClampTest : public DAIAlgFG {
    vector<Factor> _beliefsV;
    vector<Factor> _beliefsF;
    double _logZ;

    double _maxdiff;
    size_t _iters;
    
 public:
    ClampTest(const FactorGraph &fg, const PropertySet &opts) : DAIAlgFG(fg) {
        props.set(opts);
        construct();
    }
    void construct();

    static const char *Name;

    /// @name General InfAlg interface
    //@{
    virtual ClampTest* clone() const { return new ClampTest(*this); }
    virtual std::string identify() const { return string(Name) + props.toString(); }
    virtual Factor belief( const Var &n ) const { return _beliefsV[findVar(n)]; }
    virtual Factor belief( const VarSet &ns ) const;
    virtual Factor beliefV( size_t i ) const { return _beliefsV[i]; }
    virtual Factor beliefF( size_t I ) const { return _beliefsF[I]; }
    virtual std::vector<Factor> beliefs() const { return concat(_beliefsV, _beliefsF); }
    virtual Real logZ() const { return _logZ; }
    virtual void init() { }
    virtual void init( const VarSet & ) { }
    virtual double run();
    virtual double maxDiff() const { return _maxdiff; }
    virtual size_t Iterations() const { return _iters; }
    //@}

    void outputLRInfo(string file);

 public:
/* PROPERTIES(props,ClampTest) {
        /// Enumeration of clamping options: CLAMP_VAR means 'var'
        /// refers to a variable, and 'state' to a state of that
        /// variable; CLAMP_FACTOR means 'var' refers to a factor, and
        /// 'state' to a state of the factor
        DAI_ENUM(ClampType,CLAMP_VAR,CLAMP_FACTOR);
        size_t verbose;
        ClampType clamp;
        size_t var;
        size_t state;
        string infalg;
        bbp_cfn_t bbp_cfn;
        PropertySet bbp_props;
        
        /// Where to send auxiliary output (adjoints from BBP, etc.)
        string auxfile;

        /// Where to write LR info (joints and subtracted joints)
        /// (empty for nothing)
        string lr_info_file;

        /// Make this non-empty to output a (slow) plot of how the cost
        /// function changes with gradual clamping of var to state
        string clamp_plot_file;

        /// Calls numericBBPTest if true
        bool run_bbp_test;
   }
*/
/* {{{ GENERATED CODE: DO NOT EDIT. Created by 
    ./scripts/regenerate-properties include/dai/clamp_test.h src/clamp_test.cpp 
*/
    struct Properties {
        /// Enumeration of clamping options: CLAMP_VAR means 'var'
        /// refers to a variable, and 'state' to a state of that
        /// variable; CLAMP_FACTOR means 'var' refers to a factor, and
        /// 'state' to a state of the factor
        DAI_ENUM(ClampType,CLAMP_VAR,CLAMP_FACTOR);
        size_t verbose;
        ClampType clamp;
        size_t var;
        size_t state;
        string infalg;
        bbp_cfn_t bbp_cfn;
        PropertySet bbp_props;
        /// Where to send auxiliary output (adjoints from BBP, etc.)
        string auxfile;
        /// Where to write LR info (joints and subtracted joints)
/// (empty for nothing)
        string lr_info_file;
        /// Make this non-empty to output a (slow) plot of how the cost
/// function changes with gradual clamping of var to state
        string clamp_plot_file;
        /// Calls numericBBPTest if true
        bool run_bbp_test;

        /// Set members from PropertySet
        void set(const PropertySet &opts);
        /// Get members into PropertySet
        PropertySet get() const;
        /// Convert to a string which can be parsed as a PropertySet
        string toString() const;
    } props;
/* }}} END OF GENERATED CODE */
};

/// Writes a file containing, given variable i and state x_i*, the
/// list of cost function values as model is multiplied by functions
/// which change smoothly from 1 (c=0) to a delta spike at x_i = x_i*
/// (c=1)
void makeClampPlot(const InfAlg& bp, bool clampingVar,
                   size_t i, size_t xi,
                   vector<size_t> *gibbsState,
                   bbp_cfn_t cfn, const char* outfile);

}

#endif /* ____defined_libdai_clamp_test_h__ */
